@@ -1,16 +1,8 @@
 import functools
 import importlib
-import re
 
-import six
 import yaml
 
-# Python 2/3 compatibility:
-try:
-    py_string = unicode
-except NameError:  # pragma: no cover
-    py_string = str  # pragma: no cover
-
 
 def boolean(s):
     '''
@@ -37,7 +29,7 @@ def boolean(s):
 # https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md#data-types
 TYPE_MAP = {'integer': int,
             'number': float,
-            'string': py_string,
+            'string': str,
             'boolean': boolean,
             'array': list,
             'object': dict}  # map of swagger types to python types
@@ -48,13 +40,32 @@ def make_type(value, _type):
     return type_func(value)
 
 
+def deep_merge(a, b):
+    """ merges b into a
+        in case of conflict the value from b is used
+    """
+    for key in b:
+        if key in a:
+            if isinstance(a[key], dict) and isinstance(b[key], dict):
+                deep_merge(a[key], b[key])
+            elif a[key] == b[key]:
+                pass
+            else:
+                # b overwrites a
+                a[key] = b[key]
+        else:
+            a[key] = b[key]
+    return a
+
+
 def deep_getattr(obj, attr):
     """
     Recurses through an attribute chain to get the ultimate value.
-
-    Stolen from http://pingfive.typepad.com/blog/2010/04/deep-getattr-python-function.html
     """
-    return functools.reduce(getattr, attr.split('.'), obj)
+
+    attrs = attr.split('.')
+
+    return functools.reduce(getattr, attrs, obj)
 
 
 def deep_get(obj, keys):
@@ -70,9 +81,10 @@ def deep_get(obj, keys):
     """
     if not keys:
         return obj
-    try:
+
+    if isinstance(obj, list):
         return deep_get(obj[int(keys[0])], keys[1:])
-    except ValueError:
+    else:
         return deep_get(obj[keys[0]], keys[1:])
 
 
@@ -165,58 +177,30 @@ def is_null(value):
     return False
 
 
-class Jsonifier(object):
-    def __init__(self, json_):
-        self.json = json_
-
-    def dumps(self, data):
-        """ Central point where JSON serialization happens inside
-        Connexion.
-        """
-        return "{}\n".format(self.json.dumps(data, indent=2))
-
-    def loads(self, data):
-        """ Central point where JSON serialization happens inside
-        Connexion.
-        """
-        if isinstance(data, six.binary_type):
-            data = data.decode()
-
-        try:
-            return self.json.loads(data)
-        except Exception:
-            if isinstance(data, six.string_types):
-                return data
-
-
 def has_coroutine(function, api=None):
     """
     Checks if function is a coroutine.
     If ``function`` is a decorator (has a ``__wrapped__`` attribute)
     this function will also look at the wrapped function.
     """
-    if six.PY3:  # pragma: 2.7 no cover
-        import asyncio
+    import asyncio
 
-        def iscorofunc(func):
+    def iscorofunc(func):
+        iscorofunc = asyncio.iscoroutinefunction(func)
+        while not iscorofunc and hasattr(func, '__wrapped__'):
+            func = func.__wrapped__
             iscorofunc = asyncio.iscoroutinefunction(func)
-            while not iscorofunc and hasattr(func, '__wrapped__'):
-                func = func.__wrapped__
-                iscorofunc = asyncio.iscoroutinefunction(func)
-            return iscorofunc
+        return iscorofunc
 
-        if api is None:
-            return iscorofunc(function)
+    if api is None:
+        return iscorofunc(function)
 
-        else:
-            return any(
-                iscorofunc(func) for func in (
-                    function, api.get_request, api.get_response
-                )
+    else:
+        return any(
+            iscorofunc(func) for func in (
+                function, api.get_request, api.get_response
             )
-    else:  # pragma: 3 no cover
-        # there's no asyncio in python 2
-        return False
+        )
 
 
 def yamldumper(openapi):
@@ -264,12 +248,3 @@ def yamldumper(openapi):
     yaml.representer.SafeRepresenter.represent_scalar = my_represent_scalar
 
     return yaml.dump(openapi, allow_unicode=True, Dumper=NoAnchorDumper)
-
-
-def create_empty_dict_from_list(_list, _dict, _end_value):
-    """create from ['foo', 'bar'] a dict like {'foo': {'bar': {}}} recursively. needed for converting query params"""
-    current_key = _list.pop(0)
-    if _list:
-        return {current_key: create_empty_dict_from_list(_list, _dict, _end_value)}
-    else:
-        return {current_key: _end_value}
